//    CVAssistant
//    Copyright 2015-2016 Aario Shahbany
//
//    This file is part of CVAssistant.
//
//    CVAssistant is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    CVAssistant is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with CVAssistant.  If not, see <http://www.gnu.org/licenses/>.

#include "mainwindow.h"

#include <QMessageBox>
#include <QInputDialog>
#include <QBuffer>
#include <QDesktopServices>
#include <QToolButton>
#include <QDateEdit>
#include <QComboBox>
#include <QFileInfo>
#include <QProgressDialog>
#include <QScrollBar>
#if QT_VERSION < 0x050000
    #include <QUrl>
#endif
#include "models/jobapplication.h"
#include "models/abstract/field/multilinefield.h"
#include "helpers/gui.h"

#define MAX_FONT_SIZE           30
#define MIN_FONT_SIZE           10
#define MAX_FOLDED_TITLE_SIZE   100
#define DATE_FORMAT             "MM/dd/yyyy"
#define MAX_PENDING_DAYS        14
#define DONATION_DIALOG_RAND    4
//#define NO_DONATION_PROMPT

#define APP_NAME    "CV Assistant"
#define APP_URL     "http://cvassistant.sf.net"
#define APP_VERSION "3.1.0"

#include "controllers/donationdialog.h"

MainWindow::MainWindow(QWidget *parent, QString filename) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{
    ui->setupUi(this);
    qApp->processEvents();

    ui->gbPersonalInfo->layout()->addWidget(&personalInfo);
    connect(&personalInfo,SIGNAL(modified()),this,SLOT(onModified()));


    ui->wSubjects->layout()->addWidget(&subjects);
    connect(&subjects,SIGNAL(modified()),this,SLOT(onModified()));

    connect(&skills,SIGNAL(modified()),this,SLOT(onModified()));
    connect(&workExperiences,SIGNAL(modified()),this,SLOT(onModified()));
    connect(&educationDegrees,SIGNAL(modified()),this,SLOT(onModified()));
    connect(&certificates,SIGNAL(modified()),this,SLOT(onModified()));
    connect(&publications,SIGNAL(modified()),this,SLOT(onModified()));
    connect(&languages,SIGNAL(modified()),this,SLOT(onModified()));


    ui->vlTemplateTranslates->addWidget(&templateTranslates);
    connect(&templateTranslates,SIGNAL(modified()),this,SLOT(onModified()));
    ui->vlTechnicalSynonyms->addWidget(&technicalSynonyms);
    connect(&technicalSynonyms,SIGNAL(modified()),this,SLOT(onModified()));
    ui->vlCoverLetter->addWidget(&coverLetter);
    connect(&coverLetter,SIGNAL(modified()),this,SLOT(onModified()));
    ui->vlJobApplicationTracker->addWidget(&jobApplications);
    connect(&jobApplications,SIGNAL(modified()),this,SLOT(onModified()));

    gsProfilePhoto=new QGraphicsScene(ui->gvProfilePhoto);
    ui->gvProfilePhoto->setScene(gsProfilePhoto);
    profilePhotoFit=false;

    fileDialog.setFileMode(QFileDialog::AnyFile);
    fileDialog.setViewMode(QFileDialog::List);

    modified=false;
    open(filename);
}

MainWindow::~MainWindow()
{
    delete ui;

}

void MainWindow::open(QString filePath)
{
    if (!on_actionNew_triggered())
        return;
    if (filePath.isEmpty())
        return;
    qDebug() << "Reading from:\n" << filePath.toLocal8Bit().data();
    QFile file(filePath);
    if (!file.open(QFile::ReadOnly | QFile::Text)) {
        QMessageBox::warning(this, "CV Assistant",
                            "Cannot read file:\n"+filePath+"\n"+file.errorString());
        return;
    }
    QString errorStr;
    int errorLine;
    int errorColumn;
    if (!domDocument.setContent(&file, true,&errorStr,&errorLine,&errorColumn)) {
        QMessageBox::information(this, "CV Generator",
                                "Parse error at line "+QString().number(errorLine)+", column "+QString().number(errorColumn)+":\n"+errorStr);
        return;
    }
    QString fileName=QFileInfo(filePath).fileName();
    QDomElement root=domDocument.documentElement();
    QProgressDialog *pd=new QProgressDialog (this);
    pd->setMaximum(root.elementsByTagName("group").count());
    pd->setLabelText("Opening "+fileName+"\nPlease wait...");
    pd->setCancelButton(0);
    Gui::showCentered(pd, this);
    qApp->processEvents();

    QDomElement profilePhotoElementt=root.elementsByTagName("profilephoto").at(0).toElement();
    if (!profilePhotoElementt.isNull()) {
        pmProfilePhoto.loadFromData(QByteArray::fromHex(profilePhotoElementt.text().toLocal8Bit().data()),"JPG");
        gsProfilePhoto->addPixmap(pmProfilePhoto);
        qApp->processEvents();
        profilePhotoFit=false;
        on_pbZoom_clicked();
    }
    pd->setValue(1);

    QDomNodeList masterResumeNodes=root.elementsByTagName("masterresume");
    if (masterResumeNodes.count()) {
        QDomElement masterResumeElement=masterResumeNodes.at(0).toElement();
        DefaultFontSize=masterResumeElement.attribute("defaultfontsize","12").toInt();
        if (masterResumeElement.elementsByTagName(personalInfo.objectName()).count())
            personalInfo.vlayout.fromDomElement(pd,masterResumeElement.elementsByTagName(personalInfo.objectName()).at(0).toElement());
        if (masterResumeElement.elementsByTagName(subjects.objectName()).count())
            subjects.fromDomElement(pd,masterResumeElement.elementsByTagName(subjects.objectName()).at(0).toElement());
        for (int i=masterResumeNodes.at(0).childNodes().count()-1; i>0; i--) {
            QDomElement element=masterResumeNodes.at(0).childNodes().at(i).toElement();
            for (int j=0; j<ui->vlMasterResume->count(); j++) {
                GroupBox *g=static_cast<GroupBox*>(ui->vlMasterResume->itemAt(j)->widget());
                if (g->objectName()==element.nodeName()) {
                    ui->vlMasterResume->removeWidget(g);
                    ui->vlMasterResume->insertWidget(0,g);
                    g->vlayout.fromDomElement(pd,element);
                }
            }
        }
        if (masterResumeElement.elementsByTagName(technicalSynonyms.objectName()).count())
            technicalSynonyms.vlayout.fromDomElement(pd,masterResumeElement.elementsByTagName(technicalSynonyms.objectName()).at(0).toElement());
        if (masterResumeElement.elementsByTagName(templateTranslates.objectName()).count())
            templateTranslates.vlayout.fromDomElement(pd,masterResumeElement.elementsByTagName(templateTranslates.objectName()).at(0).toElement());
    }

    if (root.elementsByTagName(coverLetter.objectName()).count())
        coverLetter.vlayout.fromDomElement(pd,root.elementsByTagName(coverLetter.objectName()).at(0).toElement());

    if (root.elementsByTagName(jobApplications.objectName()).count())
        jobApplications.vlayout.fromDomElement(pd,root.elementsByTagName(jobApplications.objectName()).at(0).toElement());

    file.close();
    modified=false;
    this->filePath=filePath;
    setWindowTitle(QString("%1 %2").arg(APP_NAME,fileName));
    pd->close();
}

void MainWindow::setCheckBoxes(bool checked)
{
    QList<QCheckBox*>checkBoxes;
    if (ui->twMain->currentIndex()==0) {//Master Resume
        checkBoxes=ui->wMasterResumeParts->findChildren<QCheckBox*>(QRegExp(".*"));
        checkBoxes.append(ui->wSubjects->findChildren<QCheckBox*>(QRegExp(".*")));
    }
    else
        checkBoxes=coverLetter.findChildren<QCheckBox*>(QRegExp(".*"));
    foreach (QCheckBox* checkBox, checkBoxes)
        checkBox->setChecked(checked);
    onModified();
}

void MainWindow::onModified()
{
    if (modified)
        return; //No change to status needed
    setWindowTitle(windowTitle()+"*");
    modified=true;
}

void MainWindow::on_pbZoom_clicked()
{
    if (profilePhotoFit)
        ui->gvProfilePhoto->resetTransform();
    else
        ui->gvProfilePhoto->fitInView(gsProfilePhoto->sceneRect(),Qt::KeepAspectRatio);
    profilePhotoFit=!profilePhotoFit;
}

void MainWindow::on_pbRemoveProfilePhoto_clicked()
{
    if (gsProfilePhoto->items().count())
        gsProfilePhoto->removeItem(gsProfilePhoto->items().at(0));
    modified=true;
}

void MainWindow::on_pbAddProfilePhoto_clicked()
{
    fileDialog.setWindowTitle("Open Profile Photo");
    fileDialog.setAcceptMode(QFileDialog::AcceptOpen);
    fileDialog.setNameFilter("JPEG photo files (*.jpg);;All Files (*)");
    Gui::showCentered(&fileDialog, this);
    qApp->processEvents();
    QStringList fn;
    if (fileDialog.exec())
        fn = fileDialog.selectedFiles();
        if (!fn.count())
            return;
    qDebug() << "Loading profile photo from file:\n" << fn[0].data();
    pmProfilePhoto.load(fn[0]);
    on_pbRemoveProfilePhoto_clicked();
    gsProfilePhoto->addPixmap(pmProfilePhoto);
    qApp->processEvents();
    profilePhotoFit=false;
    on_pbZoom_clicked();
    modified=true;
}

bool MainWindow::on_actionNew_triggered()
{
    if (modified) {
        int reply= QMessageBox::question(
                    this,
                    "Resume Modified",
                    "Master Resume modified. Are you sure you want to create a new master resume without saving your changes?",
                    QMessageBox::Save | QMessageBox::Discard | QMessageBox::Cancel,
                    QMessageBox::Save);
        if (reply == QMessageBox::Save) {
            if (! on_actionSave_triggered())
                return false;
        }
        else if (reply == QMessageBox::Cancel)
            return false;
    }
    on_pbRemoveProfilePhoto_clicked();

    personalInfo.vlayout.clear();

    while (ui->vlMasterResume->count()) {
        static_cast<FieldsGroupsLayout*>(ui->vlMasterResume->itemAt(0)->widget()->layout())->clear();
        ui->vlMasterResume->removeWidget(ui->vlMasterResume->itemAt(0)->widget());
    }

    subjects.clear();
    ui->vlMasterResume->addWidget(&skills);
    skills.vlayout.checkBox.setChecked(true);
    ui->vlMasterResume->addWidget(&workExperiences);
    workExperiences.vlayout.checkBox.setChecked(true);
    ui->vlMasterResume->addWidget(&educationDegrees);
    educationDegrees.vlayout.checkBox.setChecked(true);
    ui->vlMasterResume->addWidget(&certificates);
    certificates.vlayout.checkBox.setChecked(true);
    ui->vlMasterResume->addWidget(&publications);
    publications.vlayout.checkBox.setChecked(true);
    ui->vlMasterResume->addWidget(&languages);
    languages.vlayout.checkBox.setChecked(true);

    templateTranslates.vlayout.clear();

    technicalSynonyms.vlayout.clear();

    coverLetter.vlayout.clear();

    jobApplications.vlayout.clear();

    qApp->processEvents();

    filePath="";
    modified=false;
    DefaultFontSize=12;
    setWindowTitle(QString(APP_NAME)+" Untitled");
    ui->twMain->setCurrentIndex(0);
    ui->rbJobApplicationsAll->setChecked(true);
    return true;
}

void MainWindow::showEvent(QShowEvent* event)
{
    Q_UNUSED(event)
    ui->swAnalyser->setCurrentIndex(0);
    //    this->setWindowState(Qt::WindowMaximized);
}

void MainWindow::closeEvent(QCloseEvent *event)
{
    if (!on_actionNew_triggered())
        event->ignore();
#ifndef NO_DONATION_PROMPT
    else {
        QTime time = QTime::currentTime();
        qsrand((uint)time.msec());
        int rand = qrand();
        if (rand % DONATION_DIALOG_RAND == 0) {
            DonationDialog d(this);
            d.exec();
        }
    }
#endif
}

void MainWindow::on_actionOpen_triggered()
{
    fileDialog.selectFile(filePath);
    fileDialog.setWindowTitle("Open");
    fileDialog.setAcceptMode(QFileDialog::AcceptOpen);
    fileDialog.setFileMode(QFileDialog::AnyFile);
    fileDialog.setViewMode(QFileDialog::List);
    fileDialog.setNameFilter("CV Assistant Files (*.cva);;All Files (*)");
    Gui::showCentered(&fileDialog, this);
    qApp->processEvents();
    if (!fileDialog.exec())
        return;
    QStringList fn = fileDialog.selectedFiles();
    if (fn.count())
        open(fn[0]);
}

bool MainWindow::on_actionSave_triggered()
{
    if (filePath.isEmpty())
        return on_actionSave_as_triggered();
    qDebug() << "Saving to:\n" << filePath.toLocal8Bit().data();
    domDocument.clear();
    QDomElement root=domDocument.createElement("root");
    if (gsProfilePhoto->items().count()) {
        QDomElement profilePhotoNode=domDocument.createElement("profilephoto");
        QByteArray *ba=new QByteArray();
        QBuffer bf(ba);
        bf.open(QIODevice::WriteOnly);
        pmProfilePhoto.save(&bf,"JPG");
        profilePhotoNode.appendChild(domDocument.createTextNode(ba->toHex().data()));
        root.appendChild(profilePhotoNode);
    }

    QDomElement masterResumeNode=domDocument.createElement("masterresume");
//    qDebug() << PersonalInfo.vlayout->parent()->objectName().toLocal8Bit().data();
    masterResumeNode.setAttribute("defaultfontsize",DefaultFontSize);
    masterResumeNode.appendChild(personalInfo.vlayout.toDomElement(&domDocument));
    masterResumeNode.appendChild(templateTranslates.vlayout.toDomElement(&domDocument));
    masterResumeNode.appendChild(technicalSynonyms.vlayout.toDomElement(&domDocument));
    masterResumeNode.appendChild(subjects.toDomElement(&domDocument));
    //Now we want to save each part of resume in order it is arranged in the gui
    for (int i=0; i<ui->vlMasterResume->count(); i++) {
        GroupBox *g=static_cast<GroupBox*>(ui->vlMasterResume->itemAt(i)->widget());
        masterResumeNode.appendChild(g->vlayout.toDomElement(&domDocument));
    }
    root.appendChild(masterResumeNode);
    QDomElement coverLetterNode=domDocument.createElement("coverletter");
    coverLetterNode.appendChild(coverLetter.vlayout.toDomElement(&domDocument));
    root.appendChild(coverLetterNode);
    QDomElement jobApplicationsNode=domDocument.createElement("jobapplications");
    jobApplicationsNode.appendChild(jobApplications.vlayout.toDomElement(&domDocument));
    root.appendChild(jobApplicationsNode);
    domDocument.appendChild(root);
    QFile file(filePath);
    if (!file.open(QFile::WriteOnly | QFile::Text)) {
        QMessageBox().warning(this, "CV Assistant","Cannot write file:\n"+filePath+"\n"+file.errorString()+".");
        return false;
    }
    QTextStream stream(&file);
    stream << domDocument.toString();
    file.close();
    modified=false;
    setWindowTitle(QString("%1 %2").arg(APP_NAME,QFileInfo(filePath).fileName()));
    return true;
}

bool MainWindow::on_actionSave_as_triggered()
{
    fileDialog.selectFile(filePath);
    fileDialog.setWindowTitle("Save As");
    fileDialog.setAcceptMode(QFileDialog::AcceptSave);
    fileDialog.setNameFilter("CV Assistant Files (*.cva);;All Files (*)");
    Gui::showCentered(&fileDialog, this);
    qApp->processEvents();
    if (!fileDialog.exec())
        return false;
    QStringList fn = fileDialog.selectedFiles();
    if (fn.isEmpty())
        return false;
    filePath=fn[0];
    if (filePath.split(".").count()<2)
        filePath+=".cva";
    return on_actionSave_triggered();
}

void MainWindow::on_actionExit_triggered()
{
    this->close();
}

void MainWindow::on_actionAbout_triggered()
{
    QMessageBox::about(this,
                       QString("About %1").arg(APP_NAME),
                       QString("CVAssistant\n"
                               "Version: %1\n"
                               "Build: %2-%3\n"
                               "Copyright: Aario Shahbany 2015-2035\n"
                               "Licence: GNU General Public Licence version 3\n"
                               "(GPLv3 Licence here: http://www.gnu.org/licenses/gpl-3.0.txt )")
                       .arg(APP_VERSION)
                       .arg(__DATE__)
                       .arg(__TIME__));
}

void MainWindow::on_action_Export_as_word_2007_docx_triggered()
{
    fileDialog.selectFile(filePath);
    fileDialog.setWindowTitle("Export as Word 2007 (.docx)");
    fileDialog.setAcceptMode(QFileDialog::AcceptSave);
    fileDialog.setNameFilter("Word 2007 (*.docx);;All Files (*)");
    Gui::showCentered(&fileDialog, this);
    qApp->processEvents();
    if (!fileDialog.exec())
        return;
    QStringList fn = fileDialog.selectedFiles();
    if (fn.isEmpty())
        return;
    QList<Translate*> translates=templateTranslates.translates();
    ResumeDocument Document(translates);
    Document.startNew();
    if (gsProfilePhoto->items().count())
        Document.addProfilePhoto(&pmProfilePhoto);

    Document.content+=personalInfo.vlayout.exportDocX(translates);
    Document.content+=subjects.exportDocX(translates);

    for (int i=0; i<ui->vlMasterResume->count(); i++)
        Document.content+=static_cast<GroupBox*>(ui->vlMasterResume->itemAt(i)->widget())->exportDocX(translates);
    Document.save(fn[0],DefaultFontSize);
}



void MainWindow::on_actionE_xport_Coverl_Letter_triggered()
{
    fileDialog.selectFile(filePath);
    fileDialog.setWindowTitle("Export Cover Letter as Plain Text (.txt)");
    fileDialog.setAcceptMode(QFileDialog::AcceptSave);
    fileDialog.setNameFilter("Plain Text (*.txt);;All Files (*)");
    Gui::showCentered(&fileDialog, this);
    qApp->processEvents();
    if (!fileDialog.exec())
        return;
    QStringList fn = fileDialog.selectedFiles();
    if (fn.isEmpty())
        return;

    QFile f(fn[0]);
    if (!f.open(QFile::WriteOnly | QFile::Text)) {
        QMessageBox().warning(this, "CV Assistant","Cannot write file:\n"+fn[0]+"\n"+f.errorString()+".");
        return;
    }
    QTextStream ts(&f);
    ts << coverLetter.exportText();
    f.close();
    QDesktopServices::openUrl(QUrl::fromLocalFile(fn[0]));
}

void MainWindow::on_actionResume_Analyser_triggered()
{
    ui->dwAnalyse->show();
}

void MainWindow::on_actionAnalyse_triggered()
{
    if (!RequiredSkills.count()) {
        ui->swAnalyser->setCurrentIndex(0);//Go to edit mode
        QMessageBox::warning(this, "CV Assistant",
                            "Dear Sir,\nUnfortunately I cannot find any required skills entered here.\nPlease first enter some skills line-by-line from the job advertisement.\nThen I can analyse them against your resume.\nThanks in advance!");
        return;
    }
    while (ui->wAnalyserItems->layout()->count())
        delete ui->wAnalyserItems->layout()->itemAt(0)->widget();
    qApp->processEvents();
    //First calculate rating for all required skills based on number and position of their occurance in resume
    int max=0;
    QList<int> counts;
    bool autoselect=ui->cbAnalyserAutoselect->isChecked();
    bool autoreplace=ui->cbAnalyserAutoreplace->isChecked();
    bool ask=ui->cbAskAutoreplace->isChecked();
    for (int i=0; i<RequiredSkills.count(); i++) {
        if (!RequiredSkills.at(i).count())
            continue;
        int count=0;
        QList<QStringList> synonymGroups=technicalSynonyms.synonymGroups();
        if (ui->twMain->currentIndex()==0) {//Master Resume
            count=personalInfo.vlayout.analyze(RequiredSkills.at(i),synonymGroups,autoselect,autoreplace,ask);
            count+=subjects.analyze(RequiredSkills.at(i),synonymGroups,autoselect,autoreplace,ask);
            for (int j=0; j<ui->vlMasterResume->count(); j++)
                count+=static_cast<GroupBox*>(ui->vlMasterResume->itemAt(j)->widget())->vlayout.analyze(
                            RequiredSkills.at(i),
                            synonymGroups,
                            autoselect,
                            autoreplace,
                            ask
                            );
        }
        else {//Cover Letter
            count=coverLetter.vlayout.analyze(RequiredSkills.at(i),synonymGroups,autoselect,autoreplace,ask);
        }
        if (count>max) max=count;
        counts.append(count);
    }
    //Now add labels
    for (int i=0; i<RequiredSkills.count(); i++) {
        if (!RequiredSkills.at(i).count())
            continue;
        QLabel *l=new QLabel(RequiredSkills.at(i)+QString(" (%1x)").arg(counts.at(i)));
        l->setWordWrap(true);
        QFont font;
        if (max)
            font.setPointSize(MIN_FONT_SIZE+(MAX_FONT_SIZE-MIN_FONT_SIZE)*counts.at(i)/max);
        l->setFont(font);
        l->setAlignment(Qt::AlignHCenter | Qt::AlignVCenter);
        ui->wAnalyserItems->layout()->addWidget(l);
    }
    ui->swAnalyser->setCurrentIndex(1);
}

void MainWindow::on_pbAnalyserSaveItems_clicked()
{
    if (ui->pteRequiredSkills->toPlainText().count())
        RequiredSkills=ui->pteRequiredSkills->toPlainText().split("\n");
    else
        RequiredSkills.clear();
    on_actionAnalyse_triggered();
}

void MainWindow::on_actionSelect_a_ll_triggered()
{
    setCheckBoxes(true);
}

void MainWindow::on_actionSelect_none_triggered()
{
    setCheckBoxes(false);
}

void MainWindow::on_actionHelp_triggered()
{
#if QT_VERSION >= 0x050000
    QDesktopServices::openUrl(QString(APP_URL));
#else
    QDesktopServices::openUrl(QUrl(QString(APP_URL)));
#endif
}

void MainWindow::on_actionAbout_Qt_triggered()
{
    QMessageBox::aboutQt(this);
}

void MainWindow::on_actionSet_export_font_size_triggered()
{
    DefaultFontSize=QInputDialog::getInt(this,"Font Size","Enter default font size for document",DefaultFontSize,8,24);
}

void MainWindow::on_action_Header_triggered()
{
    ui->saMasterResume->ensureWidgetVisible(ui->wMasterResumeHeader,0,600);
}

void MainWindow::on_action_Subjects_triggered()
{
    ui->saMasterResume->ensureWidgetVisible(&subjects,0,600);
}

void MainWindow::on_action_Skills_triggered()
{
    ui->saMasterResume->ensureWidgetVisible(&skills,0,600);
}

void MainWindow::on_action_Work_Experiences_triggered()
{
    ui->saMasterResume->ensureWidgetVisible(&workExperiences,0,600);
}

void MainWindow::on_actionEducation_Degrees_triggered()
{
    ui->saMasterResume->ensureWidgetVisible(&educationDegrees,0,600);
}

void MainWindow::on_actionCertificates_triggered()
{
    ui->saMasterResume->ensureWidgetVisible(&certificates,0,600);
}

void MainWindow::on_action_Publications_triggered()
{
    ui->saMasterResume->ensureWidgetVisible(&publications,0,600);
}

void MainWindow::on_action_Languages_triggered()
{
    ui->saMasterResume->ensureWidgetVisible(&languages,0,600);
}

void MainWindow::on_action_Edit_Required_Skills_triggered()
{
    ui->swAnalyser->setCurrentIndex(0);
    ui->dwAnalyse->show();
}

void MainWindow::on_action_Un_Fold_all_triggered()
{
    QWidget *currentMainWidget=ui->twMain->widget(ui->twMain->currentIndex());
    QList<FieldsGroup*> fieldsGroups=currentMainWidget->findChildren<FieldsGroup*>(QRegularExpression(".*"));
    if (fieldsGroups.isEmpty())
        return;
    bool fold=fieldsGroups.first()->foldButton.text()=="^";
    foreach(FieldsGroup* fieldsGroup, fieldsGroups)
        fieldsGroup->fold(fold);
    QList<FieldsGroupsLayout*> fieldsGroupsLayouts=currentMainWidget->findChildren<FieldsGroupsLayout*>(QRegularExpression(".*"));
    foreach(FieldsGroupsLayout* fieldsGroupsLayout, fieldsGroupsLayouts)
        fieldsGroupsLayout->fold(fold);
}

void MainWindow::on_rbJobApplicationsAll_clicked()
{
    for (int i=0; i<jobApplications.vlayout.subgroupsLayout.count(); i++)
        jobApplications.vlayout.subgroupsLayout.itemAt(i)->widget()->show();
}

void MainWindow::on_rbJobApplicationsOnGoing_clicked()
{
    for (int i=0; i<jobApplications.vlayout.subgroupsLayout.count(); i++) {
        JobApplication *jobApplication=static_cast<JobApplication*>(jobApplications.vlayout.subgroupsLayout.itemAt(i)->widget());
        jobApplication->setVisible(jobApplication->jobApplicationState()!=JobApplication::Abandoned);
    }
}

void MainWindow::on_rbJobApplicationsCritical_clicked()
{
    for (int i=0; i<jobApplications.vlayout.subgroupsLayout.count(); i++) {
        JobApplication *jobApplication=static_cast<JobApplication*>(jobApplications.vlayout.subgroupsLayout.itemAt(i)->widget());
        jobApplication->setVisible(
                    (jobApplication->jobApplicationState()==JobApplication::PendingMeCritical)
                    || (jobApplication->jobApplicationState()==JobApplication::PendingThemCritical)
                    );
    }
}

void MainWindow::on_actionArchive_Abandoned_Job_Applications_triggered()
{
    int reply= QMessageBox::question(
                this,
                "Archive Abandoned Job Applications",
                "This action archives Job Applicaitons marked as 'Abandoned' into a csv file.\n"
                "You can later open this file in a spreadsheet application.\n"
                "But it also DELETES those job applications from Job Application Tracker page."
                "This CANNOT be undone. Are you sure?",
                QMessageBox::Ok | QMessageBox::Cancel,
                QMessageBox::Cancel);
    if (reply != QMessageBox::Ok)
        return;
    fileDialog.selectFile(filePath);
    fileDialog.setWindowTitle("Archive Abandoned Job Applications (.csv)");
    fileDialog.setAcceptMode(QFileDialog::AcceptSave);
    fileDialog.setNameFilter("Comma Separated Value (*.csv);;All Files (*)");
    Gui::showCentered(&fileDialog, this);
    qApp->processEvents();
    if (!fileDialog.exec())
        return;
    QStringList fn = fileDialog.selectedFiles();
    if (fn.isEmpty())
        return;
    QString filePath=fn[0];
    QString fileName=QFileInfo(filePath).fileName();
    QFile f(filePath);
    if (!f.open(QFile::WriteOnly | QFile::Text)) {
        QMessageBox().warning(this, "CV Assistant","Cannot write file:\n"+filePath+"\n"+f.errorString()+".");
        return;
    }
    QProgressDialog *pd=new QProgressDialog (this);
    pd->setMaximum(jobApplications.vlayout.subgroupsLayout.count());
    pd->setLabelText("Archiving abandoned job applications into "+fileName+"\nPlease wait...");
    pd->setCancelButton(0);
    Gui::showCentered(pd, this);
    qApp->processEvents();
    jobApplications.archiveAbandonedApplications(pd, &f);
    f.close();
    qApp->processEvents();
    pd->close();
    qApp->processEvents();
    QDesktopServices::openUrl(QUrl::fromLocalFile(fn[0]));
}

void MainWindow::on_action_Donate_triggered()
{
    DonationDialog d(this);
    d.exec();
}

void MainWindow::on_pbJumpToTop_clicked()
{
    ui->saMasterResume->verticalScrollBar()->setValue(0);
}
